//	Dan's Color Converter - source code for the Microsoft Windows version
//	
//	Copyright 1993 by Daniel Lyke
//
//
//	This is EgoWare. You may freely copy and distribute these programs as long
//	as all parts are distributed together and unaltered. All source code may be
//	incorporated into your own programs, but I'd appreciate it if you drop me a
//	postcard or message telling me that you're using it and what you're using
//	it for. I make no warranties, express or implied, concerning the proper
//	operation of the program or any of the accompanying source code.

#define STRICT
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "color.h"


#define LOGICAL1 255

static char aachConversionTypes[][4] =
{
	"RGB", // COLORTYPE_RGB
	"CMY", // COLORTYPE_CMY
	"HLS", // COLORTYPE_HLS
	"HSV", // COLORTYPE_HSV
	"YIQ", // COLORTYPE_YIQ
	"YUV", // COLORTYPE_YUV

	"\0\0\0"
};


static char *aapszColorLabels[][3] =
{
	{ "Red", "Green", "Blue" },
	{ "Cyan", "Magenta", "Yellow" },
	{ "Hue", "Luminance", "Saturation" },
	{ "Hue", "Saturation", "Value" },
	{ "Y", "I", "Q" },
	{ "Y", "U", "V" },

	{ NULL,NULL,NULL}
};


BOOL _export FAR PASCAL ColCvtDlgProc(HWND hwnd, UINT uiMessage, UINT uiParam, LONG lParam)
{
	BOOL fRecalc;
	static int _iSourceScale = 0;
	static int _iSourceType = COLORTYPE_RGB;
	static int _iTargetScale = 0;
	static int _iTargetType = COLORTYPE_RGB;
	static int _iSourceAngle = 0;
	static int _iTargetAngle = 0;

	fRecalc = FALSE;

	switch (uiMessage)
		{
		case WM_INITDIALOG :
			SetDlgItemText(hwnd, CTRL_SOURCETEXT1, 
				aapszColorLabels[0][0]);
			SetDlgItemText(hwnd, CTRL_SOURCETEXT2, 
				aapszColorLabels[0][1]);
			SetDlgItemText(hwnd, CTRL_SOURCETEXT3, 
				aapszColorLabels[0][2]);

			CheckRadioButton(hwnd, 
				CTRL_SOURCETYPE + COLORTYPE_RGB,
				CTRL_SOURCETYPE + COLORTYPE_YUV,
				CTRL_SOURCETYPE + COLORTYPE_RGB);


			SetDlgItemText(hwnd, CTRL_TARGETTEXT1, 
				aapszColorLabels[0][0]);
			SetDlgItemText(hwnd, CTRL_TARGETTEXT2, 
				aapszColorLabels[0][1]);
			SetDlgItemText(hwnd, CTRL_TARGETTEXT3, 
				aapszColorLabels[0][2]);

			CheckRadioButton(hwnd, 
				CTRL_TARGETTYPE + COLORTYPE_RGB,
				CTRL_TARGETTYPE + COLORTYPE_YUV,
				CTRL_TARGETTYPE + COLORTYPE_RGB);

			CheckRadioButton(hwnd,
				CTRL_SOURCESCALE1,
				CTRL_SOURCESCALE255,
				CTRL_SOURCESCALE1);

			CheckRadioButton(hwnd,
				CTRL_TARGETSCALE1,
				CTRL_TARGETSCALE255,
				CTRL_TARGETSCALE1);


			CheckRadioButton(hwnd,
				CTRL_SOURCEANGLEDEFAULT,
				CTRL_SOURCEANGLEDEGREES,
				CTRL_SOURCEANGLEDEFAULT);

			CheckRadioButton(hwnd,
				CTRL_TARGETANGLEDEFAULT,
				CTRL_TARGETANGLEDEGREES,
				CTRL_TARGETANGLEDEFAULT);
			return TRUE;




		case WM_COMMAND :
			switch (uiParam)
				{
					case CTRL_SOURCETYPE + COLORTYPE_RGB :
					case CTRL_SOURCETYPE + COLORTYPE_CMY :
					case CTRL_SOURCETYPE + COLORTYPE_HLS :
					case CTRL_SOURCETYPE + COLORTYPE_HSV :
					case CTRL_SOURCETYPE + COLORTYPE_YIQ :
					case CTRL_SOURCETYPE + COLORTYPE_YUV :

						CheckRadioButton(hwnd, 
							CTRL_SOURCETYPE + COLORTYPE_RGB,
							CTRL_SOURCETYPE + COLORTYPE_YUV,
							uiParam);

						_iSourceType = uiParam - CTRL_SOURCETYPE;

						SetDlgItemText(hwnd, CTRL_SOURCETEXT1,
							aapszColorLabels[uiParam - CTRL_SOURCETYPE][0]);
						SetDlgItemText(hwnd, CTRL_SOURCETEXT2,
							aapszColorLabels[uiParam - CTRL_SOURCETYPE][1]);
						SetDlgItemText(hwnd, CTRL_SOURCETEXT3,
							aapszColorLabels[uiParam - CTRL_SOURCETYPE][2]);

						fRecalc = TRUE;
						break;

					case CTRL_TARGETTYPE + COLORTYPE_RGB :
					case CTRL_TARGETTYPE + COLORTYPE_CMY :
					case CTRL_TARGETTYPE + COLORTYPE_HLS :
					case CTRL_TARGETTYPE + COLORTYPE_HSV :
					case CTRL_TARGETTYPE + COLORTYPE_YIQ :
					case CTRL_TARGETTYPE + COLORTYPE_YUV :
						CheckRadioButton(hwnd, 
							CTRL_TARGETTYPE + COLORTYPE_RGB,
							CTRL_TARGETTYPE + COLORTYPE_YUV,
							uiParam);

						_iTargetType = uiParam - CTRL_TARGETTYPE;

						SetDlgItemText(hwnd, CTRL_TARGETTEXT1,
							aapszColorLabels[uiParam - CTRL_TARGETTYPE][0]);
						SetDlgItemText(hwnd, CTRL_TARGETTEXT2,
							aapszColorLabels[uiParam - CTRL_TARGETTYPE][1]);
						SetDlgItemText(hwnd, CTRL_TARGETTEXT3,
							aapszColorLabels[uiParam - CTRL_TARGETTYPE][2]);

						fRecalc = TRUE;
						break;

					case CTRL_SOURCECOLOR1 :
					case CTRL_SOURCECOLOR2 :
					case CTRL_SOURCECOLOR3 :
						fRecalc = TRUE;
						break;



					case CTRL_SOURCESCALE1 :
					case CTRL_SOURCESCALE100 :
					case CTRL_SOURCESCALE255 :
						CheckRadioButton(hwnd, 
							CTRL_SOURCESCALE1,
							CTRL_SOURCESCALE255,
							uiParam);

						_iSourceScale = uiParam - CTRL_SOURCESCALE1;

						fRecalc = TRUE;
						break;



					case CTRL_TARGETSCALE1 :
					case CTRL_TARGETSCALE100 :
					case CTRL_TARGETSCALE255 :
						CheckRadioButton(hwnd,
							CTRL_TARGETSCALE1,
							CTRL_TARGETSCALE255,
							uiParam);

						_iTargetScale = uiParam - CTRL_TARGETSCALE1;

						fRecalc = TRUE;
						break;


					case CTRL_SOURCEANGLEDEFAULT :
					case CTRL_SOURCEANGLERADIANS :
					case CTRL_SOURCEANGLEDEGREES :
						CheckRadioButton(hwnd, 
							CTRL_SOURCEANGLEDEFAULT,
							CTRL_SOURCEANGLEDEGREES,
							uiParam);

						_iSourceAngle = uiParam - CTRL_SOURCEANGLEDEFAULT;

						fRecalc = TRUE;
						break;



					case CTRL_TARGETANGLEDEFAULT :
					case CTRL_TARGETANGLERADIANS :
					case CTRL_TARGETANGLEDEGREES :
						CheckRadioButton(hwnd,
							CTRL_TARGETANGLEDEFAULT,
							CTRL_TARGETANGLEDEGREES,
							uiParam);

						_iTargetAngle = uiParam - CTRL_TARGETANGLEDEFAULT;

						fRecalc = TRUE;
						break;




					case CTRL_HELP :
						MessageBox(NULL,
							"Dan's Color Converter. See .DOC file for details",
							"Help",
							MB_OK);
						break;
				} // end of wm command switch
			break;


		case WM_CLOSE :
			DestroyWindow(hwnd);
			PostQuitMessage(0);
			return TRUE;

		} // end of message switch

	if (fRecalc)
		{
		TBigColor color;
		char achBuffer[255];
		double dColor1;
		double dColor2;
		double dColor3;
		double dTargetScale;
		double dSourceScale;
		double dSourceAngleUnits;
		double dTargetAngleUnits;


		if (_iSourceScale == 2)
			dSourceScale = (double)LOGICAL1;
		else if (_iSourceScale == 1)
			dSourceScale = 100.0;
		else
			dSourceScale = 1.0;

		if (_iTargetScale == 2)
			dTargetScale = (double)LOGICAL1;
		else if (_iTargetScale == 1)
			dTargetScale = 100.0;
		else
			dTargetScale = 1.0;



	   if (_iSourceAngle == 2)
			dSourceAngleUnits = 360;
		else if (_iSourceAngle == 1)
			dSourceAngleUnits = M_PI * 2;
		else
			dSourceAngleUnits = dSourceScale;


	   if (_iTargetAngle == 2)
			dTargetAngleUnits = 360;
		else if (_iTargetAngle == 1)
			dTargetAngleUnits = M_PI * 2;
		else
			dTargetAngleUnits = dTargetScale;



		
		GetDlgItemText(hwnd,CTRL_SOURCECOLOR1,achBuffer,sizeof(achBuffer) - 1);
		dColor1 = atof(achBuffer);

		GetDlgItemText(hwnd,CTRL_SOURCECOLOR2,achBuffer,sizeof(achBuffer) - 1);
		dColor2 = atof(achBuffer);

		GetDlgItemText(hwnd,CTRL_SOURCECOLOR3,achBuffer,sizeof(achBuffer) - 1);
		dColor3 = atof(achBuffer);

		if (_iSourceType == COLORTYPE_HLS ||
			_iSourceType == COLORTYPE_HSV)
			{
			EnableWindow(GetDlgItem(hwnd,CTRL_SOURCEANGLEDEFAULT),1);
			EnableWindow(GetDlgItem(hwnd,CTRL_SOURCEANGLERADIANS),1);
			EnableWindow(GetDlgItem(hwnd,CTRL_SOURCEANGLEDEGREES),1);

			color.Set(_iSourceType, 
				dColor1 / dSourceAngleUnits,
				dColor2 / dSourceScale,
				dColor3 / dSourceScale );
			}
		else
			{
			EnableWindow(GetDlgItem(hwnd,CTRL_SOURCEANGLEDEFAULT),0);
			EnableWindow(GetDlgItem(hwnd,CTRL_SOURCEANGLERADIANS),0);
			EnableWindow(GetDlgItem(hwnd,CTRL_SOURCEANGLEDEGREES),0);

			color.Set(_iSourceType,
				dColor1 / dSourceScale, 
				dColor2 / dSourceScale,
				dColor3 / dSourceScale );
			}


		dColor1 = 0.0;
		dColor2 = 0.0;
		dColor3 = 0.0;

		color.Get(_iTargetType, dColor1, dColor2, dColor3);

		if (_iTargetType == COLORTYPE_HLS ||
			_iTargetType == COLORTYPE_HSV)
			{
			EnableWindow(GetDlgItem(hwnd,CTRL_TARGETANGLEDEFAULT),1);
			EnableWindow(GetDlgItem(hwnd,CTRL_TARGETANGLERADIANS),1);
			EnableWindow(GetDlgItem(hwnd,CTRL_TARGETANGLEDEGREES),1);

			dColor1 *= dTargetAngleUnits;
			dColor2 *= dTargetScale;
			dColor3 *= dTargetScale;
			}
		else
			{
			EnableWindow(GetDlgItem(hwnd,CTRL_TARGETANGLEDEFAULT),0);
			EnableWindow(GetDlgItem(hwnd,CTRL_TARGETANGLERADIANS),0);
			EnableWindow(GetDlgItem(hwnd,CTRL_TARGETANGLEDEGREES),0);

			dColor1 *= dTargetScale;
			dColor2 *= dTargetScale;
			dColor3 *= dTargetScale;
			}


		sprintf(achBuffer,"%f",dColor1);
		SetDlgItemText(hwnd,CTRL_TARGETCOLOR1,achBuffer);

		sprintf(achBuffer,"%f",dColor2);
		SetDlgItemText(hwnd,CTRL_TARGETCOLOR2,achBuffer);

		sprintf(achBuffer,"%f",dColor3);
		SetDlgItemText(hwnd,CTRL_TARGETCOLOR3,achBuffer);

		return TRUE;
		}


	return FALSE;
}

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpszCmdParam, int nCmdShow)
{
	HWND hwnd;
	MSG msg;


	hwnd = CreateDialog(hInstance, "ClrCnv", 0, ColCvtDlgProc);
   SetClassWord(hwnd,GCW_HICON, (WORD)LoadIcon(hInstance,"ClrCvtWn"));

	while (GetMessage(&msg, NULL, 0, 0))
		{
		if (!IsDialogMessage(hwnd,&msg))
			{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			}
		}

	return 0;
} // End: WinMain
